@@ -4,7 +4,7 @@ package WebService::Avalara::AvaTax;
 
 use Modern::Perl '2011';    ## no critic (Modules::ProhibitUseQuotedVersion)
 
-our $VERSION = '0.013';     # VERSION
+our $VERSION = '0.014';     # VERSION
 use utf8;
 
 #pod =head1 SYNOPSIS
@@ -39,9 +39,12 @@ use Sys::Hostname;
 use Types::Standard qw(ArrayRef Bool HashRef InstanceOf Str);
 use WebService::Avalara::AvaTax::Service::Address;
 use WebService::Avalara::AvaTax::Service::Tax;
+use XML::LibXML::XPathContext;
 use namespace::clean;
 with 'WebService::Avalara::AvaTax::Role::Connection';
 
+const my $AVALARA_NS => 'http://avatax.avalara.com/services';
+
 #pod =method new
 #pod
 #pod Builds a new AvaTax web service client. Since this class consumes the
@@ -95,27 +98,6 @@ sub _new_service {
     );
 }
 
-#pod =head1 SEE ALSO
-#pod
-#pod =over
-#pod
-#pod =item L<Avalara Developer Network|http://developer.avalara.com/>
-#pod
-#pod Official source for Avalara developer information, including API
-#pod references, technical articles and more.
-#pod
-#pod =item L<Business::Tax::Avalara|Business::Tax::Avalara>
-#pod
-#pod An alternative that uses Avalara's REST API.
-#pod
-#pod =item L<XML::Compile::SOAP|XML::Compile::SOAP> and L<XML::Compile::WSDL11|XML::Compile::WSDL11>
-#pod
-#pod Part of the L<XML::Compile|XML::Compile> suite
-#pod and the basis for this distribution. It's helpful to understand these in
-#pod order to debug or extend this module.
-#pod
-#pod =back
-#pod
 #pod =head1 METHODS
 #pod
 #pod Aside from the L</new> method, L</services> attribute and
@@ -191,6 +173,7 @@ sub BUILD {
 
         $service->wsdl->compileCalls(
             transport => $service->_transport,
+            hooks     => $self->_make_hooks,
             %soap_params,
         );
         for my $operation ( $service->wsdl->operations(%soap_params) ) {
@@ -277,6 +260,57 @@ sub _today_to_docdate {
     return %parameters;
 }
 
+#pod =head1 WORKAROUNDS FOR INCORRECT AVALARA RESPONSES
+#pod
+#pod As of this writing the Avalara SOAP API returns responses that are
+#pod inconsistent with the WSDL document provided. Specifically, the C<TaxIncluded>
+#pod and C<GeocodeType> fields in the C<GetTaxResponse> are wrongly placed in their
+#pod sequences of fields. This module adds preprocessing hooks that attempt to
+#pod work around these problems so that the responses may be successfully parsed.
+#pod
+#pod =cut
+
+sub _make_hooks {
+    my $self = shift;
+    my $xpc  = XML::LibXML::XPathContext->new;
+    $xpc->registerNs( ava => $AVALARA_NS );
+    return [
+        {   action => 'READER',
+            type   => "{$AVALARA_NS}TaxLine",
+            before => sub {
+                my $xml = shift->cloneNode(1);
+                my $tax_included = $xpc->findnodes( 'ava:TaxIncluded', $xml );
+                $tax_included->foreach(
+                    sub {
+                        my $parent = $_->parentNode;
+                        $_->unbindNode;
+                        $parent->appendChild($_);
+                    },
+                );
+                return $xml;
+            },
+        },
+        {   action => 'READER',
+            type   => "{$AVALARA_NS}TaxAddress",
+            before => sub {
+                my $xml = shift->cloneNode(1);
+                my $geocode_type = $xpc->findnodes( 'ava:GeocodeType', $xml );
+                $geocode_type->foreach(
+                    sub {
+                        my $parent = $_->parentNode;
+                        my $validate_status
+                            = $xpc->findnodes( 'ava:ValidateStatus', $parent )
+                            ->shift;
+                        $_->unbindNode;
+                        $parent->insertAfter( $_, $validate_status );
+                    },
+                );
+                return $xml;
+            },
+        },
+    ];
+}
+
 1;
 
 __END__
@@ -294,7 +328,7 @@ WebService::Avalara::AvaTax - Avalara SOAP interface as compiled Perl methods
 
 =head1 VERSION
 
-version 0.013
+version 0.014
 
 =head1 SYNOPSIS
 
@@ -817,6 +851,16 @@ passed to both:
 When set to true at construction, the generated methods will exactly match
 their C<CamelCase> SOAP operation names.
 
+=for Pod::Coverage BUILD
+
+=head1 WORKAROUNDS FOR INCORRECT AVALARA RESPONSES
+
+As of this writing the Avalara SOAP API returns responses that are
+inconsistent with the WSDL document provided. Specifically, the C<TaxIncluded>
+and C<GeocodeType> fields in the C<GetTaxResponse> are wrongly placed in their
+sequences of fields. This module adds preprocessing hooks that attempt to
+work around these problems so that the responses may be successfully parsed.
+
 =head1 SEE ALSO
 
 =over
@@ -838,8 +882,6 @@ order to debug or extend this module.
 
 =back
 
-=for Pod::Coverage BUILD
-
 =head1 SUPPORT
 
 =head2 Perldoc
